/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2003-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.mathExpr.evaluator;

//for doc-links:
import de.jtem.mathExpr.ExpressionConfiguration;
import de.jtem.mathExpr.Function;
import de.jtem.mathExpr.parser.FunctionCall;
import de.jtem.mathExpr.parser.Symbol;


/**
 * Responsible for resolving {@link Symbol Symbols}.
 * For example a <code>Symbol</code> can be resolved to a {@link Variable}, a
 * {@link Constant} or a parameter of a function call.<br><br>
 * 
 * @see ExpressionConfiguration#defineFunction(String,String[],String) 
 * @see ExpressionConfiguration#defineVariable(String,Object,Type)
 */
public interface Context
{
  /**
   * Return the <code>Evaluator</code> for the specified <code>Symbol</code>.
   * 
   * @param s the <code>Symbol</code> an <code>Evaluator</code> will be
   * returned for.
   * @return the <code>Evaluator</code> for the specified <code>Symbol</code>.
   */
  Evaluator getEvaluator(Symbol s);
  /**
   * Return the <code>Function</code> the specified
   * <code>FunctionCall</code> is calling.
   * 
   * @param fc the <code>FunctionCall</code> calling the
   * <code>Function</code> to be return.
   * @return the <code>Function</code> the specified
   * <code>FunctionCall</code> is calling.
   */
  Function getFunction(FunctionCall fc);
}
