/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2003-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.mathExpr.parser;


/**
 * The UnaryOperation with the operator '-'. This class
 * implements the expression transformation methods 
 * {@link #simplify()} and {@link #differentiate(Symbol)} depending
 * on this certain Expression. 
 * 
 * @author marcel
 */
public class NegativeSign extends UnaryOperation
{
  /**
   * Creates a new NegativeSign object with no operand. See also constructor
   * {@link UnaryOperation#UnaryOperation(int, int)}.
   */
  public NegativeSign()
  {
    super('-', 5);
  }
  /**
   * Creates a new NegativeSign object with the specified operand. See also
   * constructor {@link
   * UnaryOperation#UnaryOperation(int, int, Expression)}.
   * 
   * @param op
   */
  public NegativeSign(Expression op)
  {
    super('-', 5, op);
  }
  /**
   * Returns a simplified Expression of this NegativeSign. Therefor this
   * method calls simplify() on its operand. Then the following
   * possible Simplifications are:
   * <ul>
   * <li>-0=0</li>
   * <li>-(-a)=a</li>
   * </ul>
   */
  public Expression simplify()
  {
    Expression newOperand=operand.simplify();
    if(newOperand instanceof NegativeSign)
    {
      UnaryOperation uo=(UnaryOperation)newOperand;
      return uo.getOperand();
    }
    if(newOperand instanceof Symbol && newOperand.isConstant())
    {
      Symbol s=(Symbol)newOperand;
      if(s.getConstantValue()==0)
        return newOperand;
    }
    return new NegativeSign(newOperand);
  }
  /**
   * Returns the derivative of this NegativeSign with respect to s.
   * In other words: Returns a new NegativeSign with this' operand
   * differentiated with respect to s.
   */
  public Expression differentiate(Symbol s)
  {
    return new NegativeSign(operand.differentiate(s)).simplify();
  }
}
